from django.core import exceptions
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_extras import forms
from django_extras.core import validators
from django_extras.core.types import Money
# Convenience Imports
from django_extras.db.models.fields.json import JsonField


class ColorField(models.CharField):
    """
    Database field that represents a color value.
    """
    default_error_messages = {
        'invalid': _(u'This value must be a CSS colour value.'),
    }
    description = _("Color value")

    def __init__(self, verbose_name=None, name=None, allow_alpha=False, **kwargs):
        kwargs.setdefault('max_length', 40)
        super(ColorField, self).__init__(verbose_name, name, **kwargs)

        self.allow_alpha = allow_alpha
        if allow_alpha:
            self.validators.append(validators.validate_alpha_color)
        else:
            self.validators.append(validators.validate_color)

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.ColorField}
        defaults.update(kwargs)
        if defaults['form_class'] is forms.ColorField:
            defaults.setdefault('allow_alpha', self.allow_alpha)
        return super(ColorField, self).formfield(**defaults)


class MoneyField(models.DecimalField):
    """
    Database field that represents a Money amount.
    """
    default_error_messages = {
        'invalid': _(u'This value must be a monetary amount.'),
        }
    description = _("Monetary amount")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_digits', 20)
        kwargs.setdefault('decimal_places', 4)
        super(MoneyField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value is None:
            return value
        try:
            return Money(value)
        except ValueError:
            raise exceptions.ValidationError(self.error_messages['invalid'])

    def get_db_prep_save(self, value, connection):
        value = self.to_python(value)
        if value is not None:
            value = value._amount
        super(MoneyField, self).get_db_prep_save(value, connection)


class PercentField(models.FloatField):
    """
    Float field that ensures field value is in the range 0-100.
    """
    default_validators = [
        MinValueValidator(0),
        MaxValueValidator(100),
        ]


class LatitudeField(models.FloatField):
    """Latitude field

    Ensures value is in hte range (-90)-90.
    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('validators', [
            validators.MinValueValidator(-90.0),
            validators.MaxValueValidator(90.0),
            ])
        super(LatitudeField, self).__init__(*args, **kwargs)


class LongitudeField(models.FloatField):
    """Longitude field

    Ensures value is in hte range (-180)-180.
    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('validators', [
            validators.MinValueValidator(-180.0),
            validators.MaxValueValidator(180.0),
            ])
        super(LongitudeField, self).__init__(*args, **kwargs)

# Register special admin widgets if admin is in use.
#if 'django.contrib.admin' in settings.INSTALLED_APPS:
#    from django.contrib.admin import options
#    options.FORMFIELD_FOR_DBFIELD_DEFAULTS[ColorField] = { 'widget':  }

# Register fields with south.
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^django_extras\.db\.models\.fields\.\w+Field"])
except ImportError:
    pass



import decimal


NO_CURRENCY_CODE = 'XXX'
NO_CURRENCY_NUMBER = 999

class Currency(object):
    """
    Represents a currency.
    """
    __slots__ = ('code', 'number', 'name', 'symbol', )

    def __init__(self, code, number, name, symbol=''):
        self.code = code
        self.number = number
        self.name = name
        self.symbol = symbol

    # Comparison operators
    def __eq__(self, other):
        if other is None:
            return False
        if isinstance(other, Currency):
            return self.code == other.code
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    @property
    def is_no_currency(self):
        return self.code == NO_CURRENCY_CODE

NoCurrency = Currency(NO_CURRENCY_CODE, NO_CURRENCY_NUMBER, 'No Currency')


def decimal_value(value):
    """
    Convert a value into a decimal and handle any conversion required.
    
    @raises ValueError if trying to convert a value that does not translate to decimal.
    """
    if value is None:
        raise ValueError('None is not a valid money value.')
    if not isinstance(value, decimal.Decimal):
        try:
            return decimal.Decimal(str(value))
        except decimal.InvalidOperation:
            raise ValueError('Value could not be converted into a decimal.')
    return value   


class Money(object):
    """
    Represents a monetary quantity.
    """
    __slots__ = ('_amount', 'currency', )

    def __init__(self, amount=decimal.Decimal('0.0'), currency=NoCurrency):
        self._amount = decimal_value(amount)
        self.currency = currency

    def __repr__(self):
        return '%5.4f' % self._amount

    def __unicode__(self):
        return self.format()

    def __hash__(self):
        return self.__repr__()

    # Math operators
    def __pos__(self):
        return Money(amount=self._amount)

    def __neg__(self):
        return Money(amount=-self._amount)

    def __add__(self, other):
        if self._can_compare(other):
            return Money(amount=self._amount + other._amount)
        else:
            return Money(amount=self._amount + decimal_value(other))

    def __sub__(self, other):
        if self._can_compare(other):
            return Money(amount=self._amount - other._amount)
        else:
            return Money(amount=self._amount - decimal_value(other))

    def __mul__(self, other):
        if self._can_compare(other):
            raise TypeError, 'Can not multiply by a monetary quantity.'
        else:
            return Money(amount = self._amount * decimal_value(other))

    def __div__(self, other):
        if self._can_compare(other):
            raise TypeError, 'Can not divide by a monetary quantity.'
        else:
            return Money(amount = self._amount * decimal_value(other))

    def __rmod__(self, other):
        """
        Re-purposed to calculate a percentage of a monetary quantity.

        >>> 10 % money.Money(500)
        50.0000
        """
        if self._can_compare(other):
            raise TypeError, 'Can not use a monetary quantity as a percentage.'
        else:
            #noinspection PyTypeChecker
            percentage = decimal_value(other) * self._amount / 100
            return Money(amount=percentage)

    def __abs__(self):
        return Money(abs(self._amount))

    __radd__ = __add__
    __rsub__ = __sub__
    __rmul__ = __mul__
    __rdiv__ = __div__

    # Comparison operators
    def __eq__(self, other):
        if other is None:
            return False
        if isinstance(other, Money):
            return self.currency == other.currency and self._amount == other._amount
        return self._amount == decimal.Decimal(str(other))

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if self._can_compare(other):
            return self._amount < other._amount
        else:
            return self._amount < decimal.Decimal(str(other))

    def __le__(self, other):
        return self < other or self == other

    def __gt__(self, other):
        if self._can_compare(other):
            return self._amount > other._amount
        else:
            return self._amount > decimal.Decimal(str(other))

    def __ge__(self, other):
        return self > other or self == other

    # Type conversion
    def __int__(self):
        return self._amount.__int__()

    def __float__(self):
        return self._amount.__float__()

    # Pickle support
    def __getstate__(self):
        return {
            'amount': self._amount,
            'currency': self.currency.code,
        }

    def __setstate__(self, state):
        self._amount = decimal_value(state.get('amount', '0.0'))

    def _can_compare(self, right):
        """
        Check if we are able to compare right value.
        
        @raises Value Error if currencies do not match.
        """
        if isinstance(right, Money):
            if self.currency == right.currency:
                return True
            else:
                raise ValueError('Currencies do not match')
        else:
            return False

    def format(self, places=2, currency_symbol='', separator=',', decimal_place='.', positive_sign='',
               negative_sign='-', trailing_negative=''):
        """
        Format money value to a string
        """
        # Round and split number
        q = decimal.Decimal(10) ** -places
        sign, digits, exp = self._amount.quantize(q).as_tuple()

        # Convert digits to string
        results = []
        digits = map(str, digits)
        build, next = results.append, digits.pop

        # Append trailing negative sign
        if sign:
            build(trailing_negative)

        # Cut off decimal digits
        if places:
            for i in range(places):
                build(next())
            build(decimal_place)

        # Append digits
        if not digits:
            build('0')
        else:
            idx = 0
            while digits:
                build(next())
                idx += 1
                if not idx % 3 and digits:
                    build(str(separator))

        # Append currency symbol and sign before returning result
        build(str(currency_symbol))
        build(str(negative_sign if sign else positive_sign))
        return ''.join(reversed(results))



from django import test
from django_extras.core import validators


class ColorValidatorTestCase(test.TestCase):
    def test_hash(self):
        validators.validate_color('#123')
        validators.validate_color('#abc')
        validators.validate_color('#45d')
        validators.validate_color('#6789ef')
        validators.validate_color('#abcdef')
        validators.validate_color('#0a1b2c')

    def test_rgb(self):
        validators.validate_color('rgb(0,1,2)')
        validators.validate_color('rgb(34,56,78)')
        validators.validate_color('rgb(191,234,156)')
        validators.validate_color('rgb(255,255,255)')

    def test_hsl(self):
        validators.validate_color('hsl(1,2%,3%)')
        validators.validate_color('hsl(23,45%,67%)')
        validators.validate_color('hsl(345,89%,90%)')
        validators.validate_color('hsl(360,100%,100%)')


class AlphaColorValidatorTestCase(test.TestCase):
    def test_hash(self):
        validators.validate_alpha_color('#123')
        validators.validate_alpha_color('#abc')
        validators.validate_alpha_color('#45d')
        validators.validate_alpha_color('#6789ef')
        validators.validate_alpha_color('#abcdef')
        validators.validate_alpha_color('#0a1b2c')

    def test_rgb(self):
        validators.validate_alpha_color('rgb(0,1,2)')
        validators.validate_alpha_color('rgb(34,56,78)')
        validators.validate_alpha_color('rgb(191,234,156)')
        validators.validate_alpha_color('rgb(255,255,255)')

    def test_rgba(self):
        validators.validate_alpha_color('rgba(0, 1, 2, 0)')
        validators.validate_alpha_color('rgba(34, 56, 78, 0.1)')
        validators.validate_alpha_color('rgba(191, 234, 156, .2)')
        validators.validate_alpha_color('rgba(255, 255, 255, 1)')

    def test_hsl(self):
        validators.validate_alpha_color('hsl(1,2%,3%)')
        validators.validate_alpha_color('hsl(23,45%,67%)')
        validators.validate_alpha_color('hsl(345,89%,90%)')
        validators.validate_alpha_color('hsl(360,100%,100%)')

    def test_hsla(self):
        validators.validate_alpha_color('hsla(1,2%,3%,0)')
        validators.validate_alpha_color('hsla(23,45%,67%,0.1)')
        validators.validate_alpha_color('hsla(345,89%,90%,.2)')
        validators.validate_alpha_color('hsla(360,100%,100%,1)')


class JsonValidatorTestCase(test.TestCase):
    def invalid_json(self):
        validators.validate_json('{"foo":"bar", "eek": 123, }')

    def test_valid(self):
        validators.validate_json('{"foo":"bar", "eek": 123}')

    def test_invalid(self):
        self.assertRaises(validators.ValidationError, self.invalid_json)




from collections import OrderedDict
from django.db.models.fields import NOT_PROVIDED


class ChoiceEnum(object):
    """
    Defines a choice set for use with Django Models.

    ChoiceEnum can be used in one of two ways, due to a side effect of how dictionaries are sorted in Python
    (dictionaries and by extension **kwargs are implemented as a hash map) the order is not preserved.

    The nicest way to use ChoiceEnum (without guaranteed ordering):

        MY_CHOICES = ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2', True), # Default, the value can be anything ;)
        )

    The less nice way to use ChoiceEnum (with guaranteed ordering):

        MY_CHOICES = ChoiceEnum(
            ('OPTION_ONE', ('value', 'Verbose value')),
            ('OPTION_TWO', ('value2', 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

    They can then be used for field choices:

        foo = models.CharField(max_length=MY_CHOICES.max_length, choices=MY_CHOICES, default=MY_CHOICES.default)

    or with the kwargs helper:

        foo = models.CharField(**MY_CHOICES.kwargs)

    And values referenced using:

        MY_CHOICES.OPTION_ONE

    Display values can be accessed via:

        MY_CHOICES.OPTION_ONE__display

    Convert a value to a display string:

        MY_CHOICES % 'value'

    """
    __slots__ = ('__choices', '__default', '__max_length', '__value_map')

    def __init__(self, *args, **entries):
        if args:
            entries = OrderedDict(args)
        if not entries:
            raise ValueError('No entries have been provided.')

        self.__default = NOT_PROVIDED
        self.__max_length = 0
        self.__value_map = dict()
        self.__choices = OrderedDict()

        map(self.__parse_entry, entries.iteritems())

    def __parse_entry(self, entry):
        key, value = entry

        if not isinstance(value, (tuple, list)):
            raise TypeError('Choice options should be a tuple or list.')
        value_len = len(value)
        if not value_len in (2, 3):
            raise ValueError('Expected choice entry in the form (Value, Verbose Value, [default]).')

        if value_len == 3:
            self.__default = value[0]
        if isinstance(value[0], basestring):
            self.__max_length = max(self.__max_length, len(value[0]))

        self.__value_map[value[0]] = value[1]
        self.__choices[key] = value[0], value[1]

    def __iter__(self):
        """
        Return choice list for use in model definition.
        """
        return iter(self.__choices.values())

    def __contains__(self, value):
        """
        Check if a choice value is in this enum.

        @param value Choice value.
        """
        return value in self.__value_map

    def __getattr__(self, item):
        """
        Get the value of an Enum.
        """
        if item.endswith('__display'):
            return self.__choices[item[:-9]][1]
        else:
            return self.__choices[item][0]

    def __mod__(self, other):
        """
        Resolve a value to it's display version.
        """
        return self.__value_map[other]

    @property
    def max_length(self):
        """
        Length of maximum value
        """
        return self.__max_length

    @property
    def default(self):
        """
        The default value.
        """
        return self.__default

    @property
    def kwargs(self):
        """
        Helper to simplify assignment of choices to a model field.
        """
        kwargs = {
            'choices': self,
            'default': self.__default,
        }
        if self.__max_length:
            kwargs['max_length'] = self.__max_length
        return kwargs

    def resolve_value(self, value):
        """
        Resolve a value to it's display version.
        """
        return self.__value_map[value]



from django import template

register = template.Library()


@register.filter
def humanize_bytes(bytes, precision=1):
    """
    Generate a humanized version of a file size.
    :param bytes:
    :param precision:
    :return:
    """
    abbrevs = (
        (1<<50L, 'PB'),
        (1<<40L, 'TB'),
        (1<<30L, 'GB'),
        (1<<20L, 'MB'),
        (1<<10L, 'kB'),
        (1, 'bytes')
    )
    if bytes == 1:
        return '1 byte'
    for factor, suffix in abbrevs:
        if bytes >= factor:
            break
    return '%.*f %s' % (precision, bytes / factor, suffix)



# Convenience import
from django.forms.widgets import *


class Html5EmailInput(TextInput):
    """
    Widget for HTML5 email input fields.
    """
    input_type = 'email'


class Html5NumberInput(TextInput):
    """
    Widget for HTML5 number input fields.
    """
    input_type = 'number'


class Html5DateInput(DateInput):
    """
    Widget for HTML5 date input fields.
    """
    input_type = 'date'


class Html5DateTimeInput(DateTimeInput):
    """
    Widget for HTML5 datetime input fields.
    """
    input_type = 'datetime'


class Html5TimeInput(TimeInput):
    """
    Widget for HTML5 time input fields.
    """
    input_type = 'time'


class JQueryColorPicker(TextInput):
    """
    Widget for jQuery UI color picker
    """
    class Media:
        css = {
            'all': ('css/jquery/ui.colorPicker.css',)
        }
        js = ('js/jquery.ui-colorPicker.js', 'js/django_extras-widgets.js',)

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        attrs['class'] = 'django_extras-colorpicker' + attrs.get('class', '')
        return super(JQueryColorPicker, self).render(name, value, attrs)



import os.path
from django.core.serializers import json
from django.utils import simplejson
from django.http import *
from email.utils import formatdate as format_http_date


class HttpResponseCreated(HttpResponse):
    status_code = 201

class HttpResponseAccepted(HttpResponse):
    status_code = 202

class HttpResponseNonAuthoritative(HttpResponse):
    status_code = 203

class HttpResponseNoContent(HttpResponse):
    status_code = 204

class HttpResponseResetContent(HttpResponse):
    status_code = 205

class HttpResponsePartialContent(HttpResponse):
    status_code = 206

# 301/302 Defined by Django

class HttpResponseSeeOther(HttpResponseRedirect):
    status_code = 303

# 304 Defined by Django

# 400 Defined by Django

class HttpResponseUnAuthorised(HttpResponse):
    status_code = 401

class HttpResponsePaymentRequired(HttpResponse):
    status_code = 402

# 403/404/405 Defined by Django

class HttpResponseNotAcceptable(HttpResponse):
    status_code = 406

class HttpResponseRequestTimeout(HttpResponse):
    status_code = 408

class HttpResponseConflict(HttpResponse):
    status_code = 409

# 410 Defined by Django

class HttpResponseLengthRequired(HttpResponse):
    status_code = 411

class HttpResponsePreconditionFailed(HttpResponse):
    status_code = 412

class HttpResponseRequestEntityTooLarge(HttpResponse):
    status_code = 413

class HttpResponseUnsupportedMediaType(HttpResponse):
    status_code = 415

class HttpResponseExpectationFailed(HttpResponse):
    status_code = 417

class HttpResponseUnprocessableEntity(HttpResponse):
    status_code = 422

class HttpResponseLocked(HttpResponse):
    status_code = 423

class HttpResponseFailedDependency(HttpResponse):
    status_code = 424

class HttpResponseUpgradeRequired(HttpResponse):
    status_code = 426

class HttpResponseNotImplemented(HttpResponse):
    status_code = 501

class HttpResponseBadGateway(HttpResponse):
    status_code = 502

class HttpResponseServiceUnavailable(HttpResponse):
    status_code = 503

class HttpResponseGatewayTimeout(HttpResponse):
    status_code = 504

class HttpResponseInsufficientStorage(HttpResponse):
    status_code = 507


class FileResponse(HttpResponse):
    """
    Response object that handles files
    """
    def __init__(self, content, content_type, include_last_modified=True):
        if isinstance(content, basestring):
            f = file(content, 'rb')
        else:
            f = content
        super(FileResponse, self).__init__(f, content_type=content_type)

        if include_last_modified:
            modified = os.path.getmtime(f.name)
            self['Last-Modified'] = format_http_date(timeval=modified, localtime=True)


class JsonResponse(HttpResponse):
    """
    Response object that handles JSON encoding and sets the correct content type.
    """
    def __init__(self, data, content_type='application/json', **kwargs):
        super(JsonResponse, self).__init__(
            simplejson.dumps(data, cls=json.DjangoJSONEncoder),
            content_type=content_type, **kwargs)



# -*- coding:utf8 -*-
__all__ = ('generate_key', 'instance_key')


def _generate_key(instance_or_type, values, postfix=None):
    opts = instance_or_type._meta
    key = 'model:%s.%s[%s]' % (
        opts.app_label, opts.module_name,
        ','.join(['%s=%s' % v for v in values])
    )
    if postfix:
        key += ':' + postfix
    return key

def generate_key(instance_or_type, *args, **kwargs):
    """
    Generate a cache/no-sql key based from a model instance or type.

    Any key/value is used to indicate a field parameter.
    """
    field_names = [f.name for f in instance_or_type._meta.fields]
    field_names.append('pk')
    for field in kwargs.iterkeys():
        if field not in field_names:
            raise AttributeError('Model "%s" has no field "%s".' % (instance_or_type._meta.module_name, field))
    return _generate_key(instance_or_type, kwargs.items(), '-'.join(args))

def instance_key(instance, fields=None, postfix=None):
    """
    Generate a cache/no-sql key based from a model instance.

    Fields can be used to generate a key not based on a private key.
    """
    if fields:
        values = [(f, getattr(instance, f)) for f in fields]
    else:
        values = [('pk', instance.pk),]
    return _generate_key(instance, values, postfix)



from django.forms.fields import *
from django_extras.forms import widgets
from django_extras.core import validators


class ColorField(CharField):
    """
    Form field that provides input for color picker
    """
    widget = widgets.JQueryColorPicker

    def __init__(self, allow_alpha=False, *args, **kwargs):
        super(ColorField, self).__init__(*args, **kwargs)
        self.allow_alpha = allow_alpha
        if allow_alpha:
            self.validators.append(validators.validate_alpha_color)
        else:
            self.validators.append(validators.validate_color)


class JsonField(Field):
    """
    Form field that validates that valid JSON is supplied.
    """
    widget = widgets.Textarea
    default_validators = [ validators.validate_json ]



# -*- encoding:utf8 -*-
from django.contrib.auth.models import *
from django.db import models


class OwnerMixinManager(models.Manager):
    """
    Manager for multiple owner mixin
    """
    def __init__(self, owner_filter):
        super(OwnerMixinManager, self).__init__()
        self.__owner_filter = owner_filter

    def _owned_by_multi(self, users):
        user_pks = [u.pk if isinstance(u, User) else u for u in users]
        filter = {self.__owner_filter + '__in': user_pks}
        return self.filter(**filter).distinct()

    def _owned_by_single(self, user, include_staff, include_superuser):
        user_pk, user = (user.pk, user) if isinstance(user, User) else (user, None)
        if include_staff or include_superuser:
            if not user:
                user = User.objects.only('is_staff', 'is_superuser').get(pk=user_pk)
            if (include_staff and user.is_staff) or (include_superuser and user.is_superuser):
                return self.all()
        filter = {self.__owner_filter: user_pk}
        return self.filter(**filter)

    def owned_by(self, user, include_staff=False, include_superuser=False):
        """
        Filter by a user(s).

        This method accepts both ``django.contrib.auth.models.User`` instances
        or user Id's, both types of value can be mixed.

        :user: user or sequence of users to also filter by. Note this requires
            use of distinct if a tuple is specified.
        :include_staff: any user who has the ``is_staff`` flag does not get
            filtered. Can only be used with a single user.
        :include_superuser: any user who has the ``is_superuser`` flag does not
            get filtered. Can only be used with a single user.
        """
        if not user:
            return []

        if isinstance(user, (list, tuple)):
            if len(user) == 1:
                # Flatten to just being a single user.
                user = user[0]
            else:
                if include_staff or include_superuser:
                    raise TypeError('Expected a User instance or int; not list or tuple.')
                return self._owned_by_multi(user)

        return self._owned_by_single(user, include_staff, include_superuser)


class OwnerMixinBase(models.Model):
    class Meta:
        abstract = True

    def get_owner_pks(self):
        """
        Get all primary keys from owners.

        This is far more efficient that loading a full user object,
         if we don't need to avoid loading full objects.

        :return: list of primary keys.
        """
        raise NotImplemented

    def is_owned_by(self, user, include_staff=False, include_superuser=False):
        """
        Is this particular model owned by a particular user.

        :user: the user object to check; this can be a ``django.contrib.auth.models.User`` instance
            or a primary key. Recommendation is to pass request.user
        :include_staff: any user who has the ``is_staff`` flag is included as an owner.
        :include_superuser: any user who has the ``is_superuser`` flag is included as an owner.
        :return: True if user has access; else False.
        """
        # Only touch elements that could cause a database operation if actually needed.
        user_pk, user = (user.pk, user) if isinstance(user, User) else (user, None)
        if include_staff or include_superuser:
            if not user:
                user = User.objects.only('is_staff', 'is_superuser').get(pk=user_pk)
            if (include_staff and user.is_staff) or (include_superuser and user.is_superuser):
                return True
        return user_pk in self.get_owner_pks()

    def is_not_owned_by(self, user, include_staff=False, include_superuser=False):
        """
        Convenience method, is an inversion of is_owned_by.
        """
        return not self.is_owned_by(user, include_staff, include_superuser)


class SingleOwnerMixin(OwnerMixinBase):
    """
    Model mixin that provides a model instance with a single owner.

    *Usage*
    ::

        class Document(SingleOwnerMixin, models.Model):
            name = models.CharField(max_length=200)
            content = models.TextField()

    """
    owner = models.ForeignKey(User, related_name='%(app_label)s_%(class)s_owner')

    objects = OwnerMixinManager('owner')

    class Meta:
        abstract = True

    def owner_list(self):
        """
        Get all owners of this model instance.

        :return: list of ``User`` instances.
        """
        return [self.owner]

    def get_owner_pks(self):
        return [self.owner_id]


class MultipleOwnerMixin(OwnerMixinBase):
    """
    Model mixin that provides a model instance with multiple owners.

    *Usage*
    ::

        class Document(MultipleOwnerMixin, models.Model):
            name = models.CharField(max_length=200)
            content = models.TextField()

    *With through class*
    ::

        class Document(MultipleOwnerMixin, models.Model):
            name = models.CharField(max_length=200)
            content = models.TextField()
            owners = models.ManyToManyField(User, through='DocumentUser')

        class DocumentUser(models.Model):
            user = models.ForeignKey(User)
            document = models.ForeignKey(Document)
            can_edit = models.BooleanField()

    """
    owners = models.ManyToManyField(User, related_name='%(app_label)s_%(class)s_owners')

    objects = OwnerMixinManager('owners')

    class Meta:
        abstract = True

    def owner_list(self):
        """
        Get all owners of this model instance.

        :return: list of ``User`` instances.
        """
        return list(self.owners.all())

    def get_owner_pks(self):
        return self.owners.values_list('id', flat=True)



import os.path
import datetime
from django import test
from django_extras.http import FileResponse, JsonResponse


class FileResponseTestCase(test.TestCase):
    def test_with_file_handle(self):
        path = os.path.dirname(__file__)
        f = file(os.path.join(path, 'data/example.txt'))
        target = FileResponse(f, 'test/plain')

        self.assertEqual(target['Content-Type'], 'test/plain')


class JsonResponseTestCase(test.TestCase):
    def test_simple_dictionary(self):
        target = JsonResponse({
            'foo': 'bar',
            'eek': 123
        })

        actual = ''.join(target)
        self.assertEqual(actual, '{"foo": "bar", "eek": 123}')
        self.assertEqual(target['Content-Type'], 'application/json')

    def test_with_dates(self):
        target = JsonResponse({
            'foo': 'bar',
            'eek': datetime.datetime(2012, 6, 25, 11, 9, 48)
        })

        actual = ''.join(target)
        self.assertEqual('{"foo": "bar", "eek": "2012-06-25 11:09:48"}', actual)
        self.assertEqual('application/json', target['Content-Type'])



from django import test
from django.db.models.fields import NOT_PROVIDED
from django_extras.db.models.choices import ChoiceEnum


class ChoicesTestCase(test.TestCase):
    def test_kwargs(self):
        target = ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2', True), # Default, the value can be anything ;)
        )

        self.assertEqual('value', target.OPTION_ONE)
        self.assertEqual('value2', target.OPTION_TWO)

    def test_args(self):
        target = ChoiceEnum(
            ('OPTION_ONE', ('value', 'Verbose value')),
            ('OPTION_TWO', ('value2', 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual('value', target.OPTION_ONE)
        self.assertEqual('value2', target.OPTION_TWO)

    def test_max_length(self):
        target = ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2', True), # Default, the value can be anything ;)
        )

        self.assertEqual(6, target.max_length)

    def test_default_set(self):
        target = ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2', True), # Default, the value can be anything ;)
        )

        self.assertEqual('value2', target.default)

    def test_default_not_set(self):
        target = ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2'),
        )

        self.assertEqual(NOT_PROVIDED, target.default)

    def test_kwargs_string_value(self):
        target = ChoiceEnum(
            ('OPTION_ONE', ('value', 'Verbose value')),
            ('OPTION_TWO', ('value2', 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual({
            'choices': target,
            'default': 'value2',
            'max_length': 6
        }, target.kwargs)

    def test_kwargs_int_value(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual({
            'choices': target,
            'default': 2
        }, target.kwargs)

    def test_resolve_value(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual('Verbose value', target.resolve_value(1))

    def test_mod(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual('Verbose value 2', target % 2)

    def test_iterator(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual([(1, 'Verbose value'), (2, 'Verbose value 2'),], list(target))

    def test_getattr(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual(1, target.OPTION_ONE)

    def test_getattr_display(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertEqual('Verbose value 2', target.OPTION_TWO__display)

    def test_contains(self):
        target = ChoiceEnum(
            ('OPTION_ONE', (1, 'Verbose value')),
            ('OPTION_TWO', (2, 'Verbose value 2', True)), # Default, the value can be anything ;)
        )

        self.assertTrue(1 in target)
        self.assertFalse(3 in target)

    def test_init_no_entries(self):
        self.assertRaises(ValueError, lambda: ChoiceEnum())

    def test_init_invalid_entry(self):
        self.assertRaises(TypeError, lambda: ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = 'value2'
        ))

        self.assertRaises(ValueError, lambda: ChoiceEnum(
            OPTION_ONE = ('value', 'Verbose value'),
            OPTION_TWO = ('value2', 'Verbose value 2', True, 1)
        ))



VERSION = (0, 2, 5, 'beta', 1)

def get_version():
    # Don't litter django_extras/__init__.py with all the get_version stuff.
    # Only import if it's actually called.
    from django_extras.utils.version import get_version
    return get_version(VERSION)



from django import test
from django_extras.forms import fields
from django_extras.core import validators

class ColorFieldTestCase(test.TestCase):
    def test_check_validator_no_alpha(self):
        target = fields.ColorField()
        self.assertIn(validators.validate_color, target.validators)

    def test_check_validator_with_alpha(self):
        target = fields.ColorField(allow_alpha=True)
        self.assertIn(validators.validate_alpha_color, target.validators)


# -*- encoding:utf8 -*-
from django.shortcuts import get_object_or_404
from django.core.exceptions import PermissionDenied


def get_owned_object_or_40x(klass, owner, include_staff=False, include_superuser=True, *args, **kwargs):
    """
    Returns an object if it can be found (using get_object_or_404).
    If the object is not owned by the supplied owner a 403 will be raised.
    """
    obj = get_object_or_404(klass, *args, **kwargs)
    if obj.is_not_owned_by(owner, include_staff, include_superuser):
        raise PermissionDenied()
    return obj



from django import test
from django.db import models
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpRequest
from django.contrib.auth.models import User
from django_extras.contrib.auth.decorators import staff_required, superuser_required
from django_extras.contrib.auth.models import SingleOwnerMixin, MultipleOwnerMixin


@staff_required
def staff_view(request, foo, bar='eek'):
    return HttpResponse('ok')

@staff_required(include_superusers=False)
def staff_only_view(request, foo, bar='eek'):
    return HttpResponse('ok')

@staff_required(raise_exception=True)
def staff_view_throw(request, foo, bar='eek'):
    return HttpResponse('ok')


class StaffRequiredTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(StaffRequiredTestCase, self).__init__(*args, **kwargs)
        self.user = User()
        self.user_staff = User(is_staff=True)
        self.user_super = User(is_superuser=True)

    def create_request(self, user):
        request = HttpRequest()
        request.user = user
        request.META['SERVER_NAME'] = 'test'
        request.META['SERVER_PORT'] = 80
        return request

    def test_normal_user(self):
        request = self.create_request(self.user)
        response = staff_view(request, 123)
        self.assertEqual(response.status_code, 302)

    def test_staff_user(self):
        request = self.create_request(self.user_staff)
        response = staff_view(request, 123)
        self.assertEqual(response.status_code, 200)

    def test_super_user(self):
        request = self.create_request(self.user_super)
        response = staff_view(request, 123)
        self.assertEqual(response.status_code, 200)

    def test_staff_only(self):
        request = self.create_request(self.user_super)
        response = staff_only_view(request, 123)
        self.assertEqual(response.status_code, 302)

    def test_raise_exception(self):
        request = self.create_request(self.user)
        self.assertRaises(PermissionDenied, lambda: staff_view_throw(request, 123))


@superuser_required
def superuser_view(request, foo, bar='eek'):
    return HttpResponse()

@superuser_required(raise_exception=True)
def superuser_view_throw(request, foo, bar='eek'):
    return HttpResponse()


class SuperuserRequiredTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(SuperuserRequiredTestCase, self).__init__(*args, **kwargs)
        self.user = User()
        self.user_staff = User(is_staff=True)
        self.user_super = User(is_superuser=True)

    def create_request(self, user):
        request = HttpRequest()
        request.user = user
        request.META['SERVER_NAME'] = 'test'
        request.META['SERVER_PORT'] = 80
        return request

    def test_normal_user(self):
        request = self.create_request(self.user)
        response = superuser_view(request, 123)
        self.assertEqual(response.status_code, 302)

    def test_staff_user(self):
        request = self.create_request(self.user_staff)
        response = superuser_view(request, 123)
        self.assertEqual(response.status_code, 302)

    def test_super_user(self):
        request = self.create_request(self.user_super)
        response = superuser_view(request, 123)
        self.assertEqual(response.status_code, 200)

    def test_raise_exception(self):
        request = self.create_request(self.user)
        self.assertRaises(PermissionDenied, lambda: superuser_view_throw(request, 123))


## Models required for the follow tests

class SingleOwner(SingleOwnerMixin, models.Model):
    name = models.CharField(max_length=50)
    description = models.TextField()

    class Meta:
        app_label = 'auth'


class MultiOwner(MultipleOwnerMixin, models.Model):
    name = models.CharField(max_length=50)
    description = models.TextField()

    class Meta:
        app_label = 'auth'


class OwnerMixinManagerTestCase(test.TransactionTestCase):
    fixtures = ['owners.json']

    def setUp(self):
        self.user1 = User.objects.get(pk=1)
        self.user2 = User.objects.get(pk=2)
        self.user_staff = User.objects.get(pk=3)
        self.user_super = User.objects.get(pk=4)

        MultiOwner.objects.create(name="Test 1", description="123").owners.add(self.user1)
        MultiOwner.objects.create(name="Test 2", description="123").owners.add(self.user1, self.user2)
        MultiOwner.objects.create(name="Test 3", description="123").owners.add(self.user2)

    def test_owned_by_none(self):
        actual = MultiOwner.objects.owned_by(None)
        self.assertSequenceEqual([], actual)

    def test_owned_by_user(self):
        actual = MultiOwner.objects.owned_by(self.user1).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)

    def test_owned_by_user_id(self):
        actual = MultiOwner.objects.owned_by(1).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)

    def test_owned_by_single_user_list(self):
        actual = MultiOwner.objects.owned_by([self.user1]).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)

    def test_owned_by_multiple_users(self):
        actual = MultiOwner.objects.owned_by([self.user1, self.user2]).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2, 3], actual)

    def test_owned_by_multiple_user_ids(self):
        actual = MultiOwner.objects.owned_by([1, 2]).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2, 3], actual)

    def test_owned_by_include_staff(self):
        actual = MultiOwner.objects.owned_by(self.user1, include_staff=True).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)

        actual = MultiOwner.objects.owned_by(self.user_staff, include_staff=True).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2, 3], actual)

        actual = MultiOwner.objects.owned_by(self.user_super, include_staff=True).values_list('id', flat=True)
        self.assertSequenceEqual([], actual)

    def test_owned_by_include_super(self):
        actual = MultiOwner.objects.owned_by(self.user1, include_superuser=True).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)

        actual = MultiOwner.objects.owned_by(self.user_staff, include_superuser=True).values_list('id', flat=True)
        self.assertSequenceEqual([], actual)

        actual = MultiOwner.objects.owned_by(self.user_super, include_superuser=True).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2, 3], actual)

    def test_owned_by_include_either_multiple(self):
        self.assertRaises(TypeError, lambda: MultiOwner.objects.owned_by([self.user_super, self.user1], include_staff=True))

    def test_owned_by_include_either_user_id(self):
        actual = MultiOwner.objects.owned_by(1, include_staff=True).values_list('id', flat=True)
        self.assertSequenceEqual([1, 2], actual)


class OwnerMixinBaseTestCase(test.TransactionTestCase):
    fixtures = ['owners.json']

    def setUp(self):
        self.user1 = User.objects.get(pk=1)
        self.user2 = User.objects.get(pk=2)
        self.user_staff = User.objects.get(pk=3)
        self.user_super = User.objects.get(pk=4)

        MultiOwner.objects.create(name="Test 1", description="123").owners.add(self.user1)
        MultiOwner.objects.create(name="Test 2", description="123").owners.add(self.user1, self.user2)
        MultiOwner.objects.create(name="Test 3", description="123").owners.add(self.user2)

    def test_is_owned_by_user(self):
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(self.user1))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(self.user2))

        self.assertTrue(MultiOwner.objects.get(pk=2).is_owned_by(self.user1))
        self.assertTrue(MultiOwner.objects.get(pk=2).is_owned_by(self.user2))

        self.assertFalse(MultiOwner.objects.get(pk=3).is_owned_by(self.user1))
        self.assertTrue(MultiOwner.objects.get(pk=3).is_owned_by(self.user2))

    def test_is_owned_by_user_id(self):
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(1))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(2))

        self.assertTrue(MultiOwner.objects.get(pk=2).is_owned_by(1))
        self.assertTrue(MultiOwner.objects.get(pk=2).is_owned_by(2))

        self.assertFalse(MultiOwner.objects.get(pk=3).is_owned_by(1))
        self.assertTrue(MultiOwner.objects.get(pk=3).is_owned_by(2))

    def test_is_owned_by_include_staff(self):
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(self.user1, include_staff=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(self.user2, include_staff=True))
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(self.user_staff, include_staff=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(self.user_super, include_staff=True))

    def test_is_owned_by_include_superuser(self):
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(self.user1, include_superuser=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(self.user2, include_superuser=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(self.user_staff, include_superuser=True))
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(self.user_super, include_superuser=True))

    def test_is_owned_by_include_with_pk(self):
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(1, include_staff=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(2, include_staff=True))
        self.assertTrue(MultiOwner.objects.get(pk=1).is_owned_by(3, include_staff=True))
        self.assertFalse(MultiOwner.objects.get(pk=1).is_owned_by(4, include_staff=True))


class SingleOwnerTestCase(test.TransactionTestCase):
    fixtures = ['owners.json']

    def setUp(self):
        self.user1 = User.objects.get(pk=1)
        self.user2 = User.objects.get(pk=2)
        self.user_staff = User.objects.get(pk=3)
        self.user_super = User.objects.get(pk=4)

        SingleOwner.objects.create(name="Test 1", description="123", owner_id=1)
        SingleOwner.objects.create(name="Test 2", description="123", owner_id=2)
        SingleOwner.objects.create(name="Test 3", description="123", owner_id=3) # Staff
        SingleOwner.objects.create(name="Test 4", description="123", owner_id=4) # Superuser

    def test_get_owner_pks(self):
        actual = SingleOwner.objects.get(pk=1).get_owner_pks()
        self.assertListEqual([1], actual)

        actual = SingleOwner.objects.get(pk=2).get_owner_pks()
        self.assertListEqual([2], actual)

    def test_get_owners_list(self):
        actual = SingleOwner.objects.get(pk=1).owner_list()
        self.assertListEqual([self.user1], actual)

        actual = SingleOwner.objects.get(pk=2).owner_list()
        self.assertListEqual([self.user2], actual)


class MultiOwnerTestTestCase(test.TransactionTestCase):
    fixtures = ['owners.json']

    def setUp(self):
        self.user1 = User.objects.get(pk=1)
        self.user2 = User.objects.get(pk=2)
        self.user_staff = User.objects.get(pk=3)
        self.user_super = User.objects.get(pk=4)

        MultiOwner.objects.create(name="Test 1", description="123").owners.add(self.user1)
        MultiOwner.objects.create(name="Test 2", description="123").owners.add(self.user1, self.user2)
        MultiOwner.objects.create(name="Test 3", description="123")
        MultiOwner.objects.create(name="Test 4", description="123")

    def test_get_owner_pks(self):
        actual = MultiOwner.objects.get(pk=1).get_owner_pks()
        self.assertSequenceEqual([1], actual)

        actual = MultiOwner.objects.get(pk=2).get_owner_pks()
        self.assertSequenceEqual([1, 2], actual)

    def test_get_owners_list(self):
        actual = MultiOwner.objects.get(pk=1).owner_list()
        self.assertListEqual([self.user1], actual)

        actual = MultiOwner.objects.get(pk=2).owner_list()
        self.assertListEqual([self.user1, self.user2], actual)


import re
from django.core.validators import *
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson


color_re = re.compile(
    r'(^#[a-f0-9]{3,6}$)' # Hash style
    r'|(^rgb\s*\(\s*((2[0-4][0-9]|25[0-5]|1?[0-9]{1,2}|100%|[0-9]{1,2}%)\s*,\s*){2}((2[0-4][0-9]|25[0-5]|1?[0-9]{1,2}|100%|[0-9]{1,2}%)\s*)\))' # rgb style
    r'|(^hsl\s*\(\s*(360|3[0-5][0-9]|[0-2]?[0-9]{1,2})\s*,\s*(100%|[0-9]{1,2}%)\s*,\s*(100%|[0-9]{1,2}%)\s*\)$)', re.IGNORECASE) # hsl style
validate_color = RegexValidator(color_re, _(u'Enter a valid color in CSS format.'), 'invalid')


alpha_color_re = re.compile(
    r'(^#[a-f0-9]{3,6}$)' # Hash style
    r'|(^rgb\s*\(\s*((2[0-4][0-9]|25[0-5]|1?[0-9]{1,2}|100%|[0-9]{1,2}%)\s*,\s*){2}((2[0-4][0-9]|25[0-5]|1?[0-9]{1,2}|100%|[0-9]{1,2}%)\s*)\))' # rgb style
    r'|(^rgba\s*\(\s*((2[0-4][0-9]|25[0-5]|1?[0-9]{1,2}|100%|[0-9]{1,2}%)\s*,\s*){3}(((0?(\.[0-9]+)?)|1)\s*)\)$)' # rgba style
    r'|(^hsl\s*\(\s*(360|3[0-5][0-9]|[0-2]?[0-9]{1,2})\s*,\s*(100%|[0-9]{1,2}%)\s*,\s*(100%|[0-9]{1,2}%)\s*\)$)' # hsl style
    r'|(^hsla\s*\(\s*(360|3[0-5][0-9]|[0-2]?[0-9]{1,2})\s*,\s*((100%|[0-9]{1,2}%)\s*,\s*){2}(((0?(\.[0-9]+)?)|1)\s*)\)$)', re.IGNORECASE) # hsla style
validate_alpha_color = RegexValidator(alpha_color_re, _(u'Enter a valid color in CSS format.'), 'invalid')


class JsonValidator(object):
    message = _(u'Enter valid JSON.')
    code = 'invalid'

    def __init__(self, message=None, code=None, load_options=None):
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code
        self.load_options = load_options if load_options else {}

    def __call__(self, value):
        """
        Validates that the input is valid JSON.
        """
        try:
            simplejson.loads(value, **self.load_options)
        except ValueError:
            raise ValidationError(self.message, code=self.code)

validate_json = JsonValidator()



from django_extras.tests.contrib.auth import *
from django_extras.tests.core.types import *
from django_extras.tests.core.validators import *
from django_extras.tests.db.choices import *
from django_extras.tests.forms.fields import *
from django_extras.tests.http.responses import *
from django_extras.tests.middleware.timing import *
from django_extras.tests.utils.cache import *



# -*- encoding:utf8 -*-
from django.contrib.auth.decorators import *
from django.core.exceptions import PermissionDenied


def superuser_required(function=None, login_url=None, raise_exception=False):
    """
    Decorator for views that checks that the user is a superuser, redirecting
    to the log-in page if necessary.
    """
    def check_permission(user):
        if user.is_superuser:
            return True
        if raise_exception:
            raise PermissionDenied
        return False
    actual_decorator = user_passes_test(check_permission, login_url=login_url)
    if function:
        return actual_decorator(function)
    return actual_decorator


def staff_required(function=None, include_superusers=True, login_url=None,
                   raise_exception=False):
    """
    Decorator for views that checks that the user is a staff member,
    redirecting to the log-in page if necessary.
    """
    def check_permission(user):
        if user.is_staff or (include_superusers and user.is_superuser):
            return True
        if raise_exception:
            raise PermissionDenied
        return False
    actual_decorator = user_passes_test(check_permission, login_url=login_url)
    if function:
        return actual_decorator(function)
    return actual_decorator



#coding=UTF-8
from django import test
from django_extras.core.types import Money


#TODO: Add more tests to get complete coverage
class MoneyTestCase(test.TestCase):

    LARGE = Money('200000000.0000')
    SMALL = Money('100.000')
    NEGATIVE = Money('-200000.0000')

    FORMAT_POSITIVE = Money('123456.789')
    FORMAT_NEGATIVE = Money('-12345.6789')

    def test_format_default(self):
        self.assertEqual('123,456.79', self.FORMAT_POSITIVE.format())
        self.assertEqual('-12,345.68', self.FORMAT_NEGATIVE.format())

    def test_format_places(self):
        self.assertEqual('123,456.7890', self.FORMAT_POSITIVE.format(places=4))
        self.assertEqual('-12,345.678900', self.FORMAT_NEGATIVE.format(places=6))

    def test_format_currency_symbol(self):
        self.assertEqual('$123,456.79', self.FORMAT_POSITIVE.format(currency_symbol='$'))
        self.assertEqual('-£12,345.68', self.FORMAT_NEGATIVE.format(currency_symbol='£'))

    def test_format_europe(self):
        self.assertEqual('123.456,79', self.FORMAT_POSITIVE.format(separator='.', decimal_place=','))
        self.assertEqual('-12.345,68', self.FORMAT_NEGATIVE.format(separator='.', decimal_place=','))

    def test_format_signs(self):
        self.assertEqual('p123,456.79', self.FORMAT_POSITIVE.format(positive_sign="p"))
        self.assertEqual('n12,345.68', self.FORMAT_NEGATIVE.format(negative_sign="n"))

    def test_format_trailing_negative(self):
        self.assertEqual('123,456.79', self.FORMAT_POSITIVE.format(trailing_negative=" neg"))
        self.assertEqual('-12,345.68 neg', self.FORMAT_NEGATIVE.format(trailing_negative=" neg"))




from decimal import Decimal
from django.core.serializers.json import DjangoJSONEncoder
from django.conf import settings
from django.db import models
from django.utils import simplejson
from django_extras import forms


def dumps(value):
    return DjangoJSONEncoder().encode(value)


def loads(txt):
    value = simplejson.loads(
        txt,
        parse_float=Decimal,
        encoding=settings.DEFAULT_CHARSET
    )
    return value


class JsonDict(dict):
    """
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return dumps(self)


class JsonList(list):
    """
    As above
    """
    def __repr__(self):
        return dumps(self)


class JsonField(models.TextField):
    """Field that serializes/de-serializes a python list/dictionary to the
    database seamlessly."""

    # Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = '{}'
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""
        if value is None or value == '':
            return {}
        elif isinstance(value, basestring):
            res = loads(value)
            if isinstance(res, dict):
                return JsonDict(**res)
            else:
                return JsonList(res)
        else:
            return value

    def get_db_prep_save(self, value, connection):
        """Convert our JSON object to a string before we save"""
        if not isinstance(value, (list, dict)):
            return super(JsonField, self).get_db_prep_save("", connection=connection)
        else:
            return super(JsonField, self).get_db_prep_save(dumps(value),
                connection=connection)

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.JsonField}
        defaults.update(kwargs)
        return super(JsonField, self).formfield(**defaults)


# Register field with south.
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^django_extras\.db\.models\.fields\.json\.JsonField"])
except ImportError:
    pass



from django import test
from django.db import models
from django_extras.utils.cache import generate_key, instance_key


class TestModel(models.Model):
    name = models.CharField(max_length=10)

    class Meta:
        app_label = 'test'


class GenerateKeyTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(GenerateKeyTestCase, self).__init__(*args, **kwargs)
        self.m = TestModel(id=1, name='eek')

    def test_type_using_pk(self):
        actual = generate_key(TestModel, pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]')

    def test_type_using_knowen_field(self):
        actual = generate_key(TestModel, id=2)
        self.assertEqual(actual, 'model:test.testmodel[id=2]')

    def test_type_using_unknown_field(self):
        self.assertRaises(AttributeError, lambda : generate_key(TestModel, eek=2))

    def test_type_using_single_postfix(self):
        actual = generate_key(TestModel, 'cover', pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]:cover')

    def test_type_using_multiple_postfix(self):
        actual = generate_key(TestModel, 'foo', 'bar', pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]:foo-bar')

    def test_instance_using_pk(self):
        actual = generate_key(self.m, pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]')

    def test_instance_using_known_field(self):
        actual = generate_key(self.m, id=2)
        self.assertEqual(actual, 'model:test.testmodel[id=2]')

    def test_instance_using_unknown_field(self):
        self.assertRaises(AttributeError, lambda : generate_key(self.m, eek=2))

    def test_instance_using_single_postfix(self):
        actual = generate_key(self.m, 'cover', pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]:cover')

    def test_instance_using_multiple_postfix(self):
        actual = generate_key(self.m, 'foo', 'bar', pk=1)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]:foo-bar')


class InstanceKeyTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(InstanceKeyTestCase, self).__init__(*args, **kwargs)
        self.m = TestModel(id=1, name='eek')

    def test_just_instance(self):
        actual = instance_key(self.m)
        self.assertEqual(actual, 'model:test.testmodel[pk=1]')

    def test_specific_fields(self):
        actual = instance_key(self.m, ('name', ))
        self.assertEqual(actual, 'model:test.testmodel[name=eek]')

    def test_unknown_field(self):
        self.assertRaises(AttributeError, lambda : instance_key(self.m, ('eek', )))

    def test_postfix(self):
        actual = instance_key(self.m, ('name', ), 'cover')
        self.assertEqual(actual, 'model:test.testmodel[name=eek]:cover')



from setuptools import setup
import os

# Hack to force dist utils to install data files in correct location along with
# Python package.
from distutils.command.install import INSTALL_SCHEMES
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)
    
# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
django_extras_dir = 'django_extras'

for dirpath, dirnames, filenames in os.walk(django_extras_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

# Dynamically calculate the version based on django_extras.VERSION.
version = __import__('django_extras').get_version()

setup(
    name = "django-extras",
    version = version,
    url = "https://bitbucket.org/timsavage/django-extras",
    author = "Tim Savage",
    author_email = "tim.savage@poweredbypenguins.org",
    description = "A selection of extras for the Django Web framework.",
    packages = packages,
    data_files = data_files,
    requires = ['django'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import datetime


class TimingMiddleware(object):
    """
    Appends the X-PROCESSING_TIME_MS header to all responses.

    This value is the total time spent processing a user request in microseconds.
    """
    REQUEST_ATTR = '_timing_start'
    RESPONSE_HEADER = 'X-PROCESSING_TIME_MS'

    def process_request(self, request):
        setattr(request, self.REQUEST_ATTR, datetime.datetime.now())

    def process_response(self, request, response):
        start = getattr(request, self.REQUEST_ATTR, None)
        if start:
            end = datetime.datetime.now()
            length = end - start
            response[self.RESPONSE_HEADER] = str(length.microseconds)
        return response
    


from django import test
from django.http import HttpRequest, HttpResponse
from django_extras.middleware.timing import TimingMiddleware


class TimingMiddlewareTestCase(test.TestCase):
    def test_full_lifecycle(self):
        request = HttpRequest()
        response = HttpResponse()
        target = TimingMiddleware()

        target.process_request(request)
        assert(hasattr(request, TimingMiddleware.REQUEST_ATTR))

        target.process_response(request, response)
        self.assertIsNotNone(response[TimingMiddleware.RESPONSE_HEADER])

    def test_process_response_only(self):
        request = HttpRequest()
        response = HttpResponse()
        target = TimingMiddleware()

        target.process_response(request, response)
        self.assertRaises(KeyError, lambda: response[TimingMiddleware.RESPONSE_HEADER])
